@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Void
@@ -9,7 +10,7 @@
 -- Portability :  portable
 --
 ----------------------------------------------------------------------------
-module Data.Void (Void, absurd, vacuous) where
+module Data.Void (Void, absurd, vacuous, vacuousM) where
 
 import Data.Semigroup (Semigroup(..))
 import Data.Ix
@@ -20,33 +21,48 @@ import Data.Data
 
 #ifdef __GLASGOW_HASKELL__
 import Unsafe.Coerce
+#else
+import Control.Monad (liftM)
 #endif
 
+-- | A logically uninhabited data type.
 #if __GLASGOW_HASKELL__ < 700
-data Void = Void !Void 
+data Void = Void !Void
 #else
 newtype Void = Void Void
 #endif
-  deriving 
+  deriving
   ( Eq, Ord, Show, Read
 #ifdef LANGUAGE_DeriveDataTypeable
   , Data, Typeable
 #endif
   )
 
--- | Since Void values are logically uninhabited, this witnesses the logical
--- reasoning tool of 'ex falso quodlibet'.
+-- | Since 'Void' values logically don't exist, this witnesses the logical
+-- reasoning tool of \"ex falso quodlibet\".
 absurd :: Void -> a
-absurd (Void a) = absurd a
+absurd a = a `seq` spin a where
+   spin (Void b) = spin b
 
+-- | If 'Void' is uninhabited then any 'Functor' that holds only values of type 'Void'
+-- is holding no values.
 vacuous :: Functor f => f Void -> f a
 #ifdef __GLASGOW_HASKELL__
 vacuous = unsafeCoerce
 #else
--- other haskell compilers are free to use less homogeneous representations
+-- other haskell compilers are free to use less homogeneous representations (NHC does, for instance)
 vacuous = fmap absurd
 #endif
 
+-- | If 'Void' is uninhabited then any 'Monad' that holds values of type 'Void'
+-- is holding no values.
+vacuousM :: Monad m => m Void -> m a
+#ifdef __GLASGOW_HASKELL__
+vacuousM = unsafeCoerce
+#else
+vacuousM = liftM absurd
+#endif
+
 instance Semigroup Void where
   a <> _ = a
   times1p _ a = a
@@ -55,4 +71,4 @@ instance Ix Void where
   range _ = []
   index _ = absurd
   inRange _ = absurd
-  rangeSize _ = 0 
+  rangeSize _ = 0
